package com.css.simulation.resource.scheduler.service;

import api.common.util.SshUtil;
import com.css.simulation.resource.scheduler.manager.TaskManager;
import com.css.simulation.resource.scheduler.mapper.IndexTemplateMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import io.minio.MinioClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.session.ClientSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class TaskService {


    @Autowired
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    TaskManager taskManager;
    @Autowired
    IndexTemplateMapper indexTemplateMapper;
    @Value("${scheduler.manual-project.topic}")
    String manualProjectTopic;
    @Value("${scheduler.score.hostname}")
    String hostname;
    @Value("${scheduler.score.username}")
    String username;
    @Value("${scheduler.score.password}")
    String password;
    @Value("${spring.kafka.delete-command}")
    String kafkaDeleteCommand;
    @Autowired
    MinioClient minioClient;
    @Value("${minio.bucket-name}")
    String bucketName;
    @Autowired
    TaskMapper taskMapper;


    @SneakyThrows
    public void taskState(String taskId, String state, String podName) {
        log.info("TaskService--state 接收到参数为：taskId=" + taskId + "，state=" + state + "，podName=" + podName);
        TaskPO taskPO = taskMapper.selectById(taskId);
        if (taskPO == null) {
            log.error("TaskManager--isProjectCompleted 接收到已删除但还在执行的任务：taskId=" + taskId + "，state=" + state + "，podName=" + podName);
            return;
        }
        String projectId = taskPO.getPId();
        String userId = taskPO.getCreateUserId();
        SshClient client = SshUtil.getClient();
        ClientSession session = SshUtil.getSession(client, hostname, username, password);
        //1 判断项目是否已完成
        boolean projectCompleted = taskManager.isProjectCompleted(userId, projectId, taskId, state, podName, session);
        if (!projectCompleted) {
            session.close();
            client.stop();
            return;
        }

        //2 准备打分
        taskManager.prepareScore(userId, projectId);

        //3 打分
        taskManager.score(userId, projectId,session);


        // -------------------------------- 收尾 --------------------------------

        // 删除所有 key
//        Set<String> keys = redisTemplate.keys("manualProject:" + projectId + "*");
//        assert keys != null;
//        redisTemplate.delete(keys);
//        log.info("------- /state 任务 " + taskId + " 的父项目为：" + projectId);


        // 删除 kafka topic
//        SshClient clientKafka = SshUtil.getClient();
//        ClientSession sessionKafka = SshUtil.getSession(clientKafka, hostnameKafka, usernameKafka, passwordKafka);
//        String topicDeleteCommand = StringUtil.replace(kafkaDeleteCommand, "topicName", projectId);
//        SshUtil.execute(sessionKafka, topicDeleteCommand);
//        SshUtil.stop(clientKafka, sessionKafka);


        // 删除 job
        SshUtil.execute(session, "kubectl delete job project-" + projectId);
        session.close();
        client.stop();

    }

    public Boolean taskConfirm(String taskId) {
        return taskManager.taskConfirm(taskId);
    }

    public void taskTick(String taskId) {
        taskManager.taskTick(taskId);
    }


}



